import { WebLocksMixin } from './chunk-JKVIPQ3X.js';
import { wa_sqlite_async_default } from './chunk-5LYQPZP3.js';
import { FacadeVFS } from './chunk-BESRLAWD.js';
import { SQLITE_OPEN_CREATE, SQLITE_OK, SQLITE_CANTOPEN, SQLITE_IOERR_DELETE, SQLITE_IOERR_ACCESS, SQLITE_OPEN_DELETEONCLOSE, SQLITE_IOERR_SHORT_READ, SQLITE_IOERR_READ, SQLITE_IOERR_WRITE, SQLITE_IOERR_TRUNCATE, SQLITE_IOERR_FSYNC, SQLITE_IOERR_FSTAT, SQLITE_LOCK_SHARED } from './chunk-6A5XENKU.js';

// src/vfs/class/OPFSAnyContextVFS.js
var File = class {
  /** @type {string} */
  pathname;
  /** @type {number} */
  flags;
  /** @type {FileSystemFileHandle} */
  fileHandle;
  /** @type {Blob?} */
  blob;
  /** @type {FileSystemWritableFileStream?} */
  writable;
  constructor(pathname, flags) {
    this.pathname = pathname;
    this.flags = flags;
  }
};
var OPFSAnyContextVFS = class _OPFSAnyContextVFS extends WebLocksMixin(FacadeVFS) {
  /** @type {Map<number, File>} */
  mapIdToFile = /* @__PURE__ */ new Map();
  lastError = null;
  log = null;
  /**
   * HACK: root directory handle, to match `isFsHandleVFS` check
  */
  releaser = null;
  /**
   * @param {string} path
   * @param {boolean|undefined} create
   * @returns {Promise<[FileSystemDirectoryHandle, string]>}
   */
  async getPathComponents(path, create) {
    const [_, directories, filename] = new URL(path, "file://").pathname.match(/\/?(.*)\/(.*)$/);
    let directoryHandle = this.releaser;
    for (const directory of directories.split("/")) {
      if (directory) {
        directoryHandle = await directoryHandle.getDirectoryHandle(directory, { create });
      }
    }
    return [directoryHandle, filename];
  }
  static async create(name, module, options, root) {
    const vfs = new _OPFSAnyContextVFS(name, module, options);
    vfs.releaser = root;
    await vfs.isReady();
    return vfs;
  }
  constructor(name, module, options = {}) {
    super(name, module, options);
  }
  getFilename(fileId) {
    const pathname = this.mapIdToFile.get(fileId).pathname;
    return `OPFS:${pathname}`;
  }
  /**
   * @param {string?} zName
   * @param {number} fileId
   * @param {number} flags
   * @param {DataView} pOutFlags
   * @returns {Promise<number>}
   */
  async jOpen(zName, fileId, flags, pOutFlags) {
    try {
      const p = zName || Math.random().toString(36).slice(2);
      const file = new File(p, flags);
      this.mapIdToFile.set(fileId, file);
      const create = !!(flags & SQLITE_OPEN_CREATE);
      const [directoryHandle, filename] = await this.getPathComponents(p, create);
      file.fileHandle = await directoryHandle.getFileHandle(filename, { create });
      pOutFlags.setInt32(0, flags, true);
      return SQLITE_OK;
    } catch (e) {
      this.lastError = e;
      return SQLITE_CANTOPEN;
    }
  }
  /**
   * @param {string} zName
   * @param {number} syncDir
   * @returns {Promise<number>}
   */
  async jDelete(zName, syncDir) {
    try {
      const [directoryHandle, name] = await this.getPathComponents(zName);
      const result = directoryHandle.removeEntry(name, { recursive: false });
      if (syncDir) {
        await result;
      }
      return SQLITE_OK;
    } catch (e) {
      return SQLITE_IOERR_DELETE;
    }
  }
  /**
   * @param {string} zName
   * @param {number} flags
   * @param {DataView} pResOut
   * @returns {Promise<number>}
   */
  async jAccess(zName, flags, pResOut) {
    try {
      const [directoryHandle, dbName] = await this.getPathComponents(zName);
      const fileHandle = await directoryHandle.getFileHandle(dbName, { create: false });
      pResOut.setInt32(0, 1, true);
      return SQLITE_OK;
    } catch (e) {
      if (e.name === "NotFoundError") {
        pResOut.setInt32(0, 0, true);
        return SQLITE_OK;
      }
      this.lastError = e;
      return SQLITE_IOERR_ACCESS;
    }
  }
  /**
   * @param {number} fileId
   * @returns {Promise<number>}
   */
  async jClose(fileId) {
    try {
      const file = this.mapIdToFile.get(fileId);
      this.mapIdToFile.delete(fileId);
      await file.writable?.close();
      if (file?.flags & SQLITE_OPEN_DELETEONCLOSE) {
        const [directoryHandle, name] = await this.getPathComponents(file.pathname);
        await directoryHandle.removeEntry(name, { recursive: false });
      }
      return SQLITE_OK;
    } catch (e) {
      return SQLITE_IOERR_DELETE;
    }
  }
  /**
   * @param {number} fileId
   * @param {Uint8Array} pData
   * @param {number} iOffset
   * @returns {Promise<number>}
   */
  async jRead(fileId, pData, iOffset) {
    try {
      const file = this.mapIdToFile.get(fileId);
      if (file.writable) {
        await file.writable.close();
        file.writable = null;
        file.blob = null;
      }
      if (!file.blob) {
        file.blob = await file.fileHandle.getFile();
      }
      const bytesRead = await file.blob.slice(iOffset, iOffset + pData.byteLength).arrayBuffer().then((arrayBuffer) => {
        pData.set(new Uint8Array(arrayBuffer));
        return arrayBuffer.byteLength;
      });
      if (bytesRead < pData.byteLength) {
        pData.fill(0, bytesRead);
        return SQLITE_IOERR_SHORT_READ;
      }
      return SQLITE_OK;
    } catch (e) {
      this.lastError = e;
      return SQLITE_IOERR_READ;
    }
  }
  /**
   * @param {number} fileId
   * @param {Uint8Array} pData
   * @param {number} iOffset
   * @returns {Promise<number>}
   */
  async jWrite(fileId, pData, iOffset) {
    try {
      const file = this.mapIdToFile.get(fileId);
      if (!file.writable) {
        file.writable = await file.fileHandle.createWritable({ keepExistingData: true });
      }
      await file.writable.seek(iOffset);
      await file.writable.write(pData.subarray());
      file.blob = null;
      return SQLITE_OK;
    } catch (e) {
      this.lastError = e;
      return SQLITE_IOERR_WRITE;
    }
  }
  /**
   * @param {number} fileId
   * @param {number} iSize
   * @returns {Promise<number>}
   */
  async jTruncate(fileId, iSize) {
    try {
      const file = this.mapIdToFile.get(fileId);
      if (!file.writable) {
        file.writable = await file.fileHandle.createWritable({ keepExistingData: true });
      }
      await file.writable.truncate(iSize);
      file.blob = null;
      return SQLITE_OK;
    } catch (e) {
      this.lastError = e;
      return SQLITE_IOERR_TRUNCATE;
    }
  }
  /**
   * @param {number} fileId
   * @param {number} flags
   * @returns {Promise<number>}
   */
  async jSync(fileId, flags) {
    try {
      const file = this.mapIdToFile.get(fileId);
      await file.writable?.close();
      file.writable = null;
      file.blob = null;
      return SQLITE_OK;
    } catch (e) {
      this.lastError = e;
      return SQLITE_IOERR_FSYNC;
    }
  }
  /**
   * @param {number} fileId
   * @param {DataView} pSize64
   * @returns {Promise<number>}
   */
  async jFileSize(fileId, pSize64) {
    try {
      const file = this.mapIdToFile.get(fileId);
      if (file.writable) {
        await file.writable.close();
        file.writable = null;
        file.blob = null;
      }
      if (!file.blob) {
        file.blob = await file.fileHandle.getFile();
      }
      pSize64.setBigInt64(0, BigInt(file.blob.size), true);
      return SQLITE_OK;
    } catch (e) {
      this.lastError = e;
      return SQLITE_IOERR_FSTAT;
    }
  }
  /**
   * @param {number} fileId
   * @param {number} lockType
   * @returns {Promise<number>}
   */
  async jLock(fileId, lockType) {
    if (lockType === SQLITE_LOCK_SHARED) {
      const file = this.mapIdToFile.get(fileId);
      file.blob = null;
    }
    return super.jLock(fileId, lockType);
  }
  jGetLastError(zBuf) {
    if (this.lastError) {
      console.error(this.lastError);
      const outputArray = zBuf.subarray(0, zBuf.byteLength - 1);
      const { written } = new TextEncoder().encodeInto(this.lastError.message, outputArray);
      zBuf[written] = 0;
    }
    return SQLITE_OK;
  }
};

// src/vfs/fs-handle.ts
async function useFsHandleStorage(path, rootDirHandle, options = {}) {
  const { url, ...rest } = options;
  const sqliteModule = await wa_sqlite_async_default(
    options.url ? { locateFile: () => options.url } : void 0
  );
  return {
    path,
    sqliteModule,
    vfsFn: (...args) => OPFSAnyContextVFS.create(...args, rootDirHandle),
    ...rest
  };
}

export { OPFSAnyContextVFS, useFsHandleStorage };
