import { P as Promisable, I as InitSQLiteOptions, S as SQLiteDB, a as SQLiteDBCore, F as FacadeVFS, B as BaseStorageOptions } from './index-DoihFDbg.cjs';
export { b as IDBBatchAtomicVFSOptions } from './index-DoihFDbg.cjs';

/**
 * Load SQLite database, presets:
 * - `useMemoryStorage`
 * - `useIdbStorage`
 * - `useIdbMemoryStorage`
 * - `useOpfsStorage`
 * - `useFsHandleStorage`
 * @param options {@link InitSQLiteOptions}
 */
declare function initSQLite(options: Promisable<InitSQLiteOptions>): Promise<SQLiteDB>;
/**
 * Load SQLite database without utils
 *
 * Presets: `useMemoryStorage`, `useIdbStorage`, `useOpfsStorage`
 * @param options {@link InitSQLiteOptions}
 */
declare function initSQLiteCore(options: Promisable<InitSQLiteOptions>): Promise<SQLiteDBCore>;

declare function dumpVFS(vfs: FacadeVFS, path: string, onDone?: (vfs: FacadeVFS, path: string) => any): ReadableStream;
declare function exportDatabaseFromIDB(vfs: FacadeVFS, path: string): Promise<Uint8Array>;
declare function exportDatabaseFromFsHandle(vfs: FacadeVFS, path: string): Promise<Uint8Array>;
/**
 * Export database to `Uint8Array`
 * @param vfs SQLite VFS
 * @param path database path
 */
declare function exportDatabase(vfs: FacadeVFS, path: string): Promise<Uint8Array>;

declare function importDatabaseToIdb(vfs: FacadeVFS, path: string, stream: ReadableStream<Uint8Array>): Promise<void>;
/**
 * Import database from `File` or `ReadableStream`
 * @param vfs SQLite VFS
 * @param path db path
 * @param data existing database
 */
declare function importDatabase(vfs: FacadeVFS, path: string, data: File | ReadableStream<Uint8Array>): Promise<void>;

/**
 * check if IndexedDB and Web Locks API supported
 */
declare function isIdbSupported(): boolean;
/**
 * check if [OPFS SyncAccessHandle](https://developer.mozilla.org/en-US/docs/Web/API/FileSystemSyncAccessHandle) supported
 */
declare function isOpfsSupported(): Promise<boolean>;
/**
 * check `new Worker(url, { type: 'module' })` support
 *
 * {@link https://stackoverflow.com/questions/62954570/javascript-feature-detect-module-support-for-web-workers Reference}
 */
declare function isModuleWorkerSupport(): boolean;
/**
 * Create custom function, run js script in SQLite
 *
 * @example
 * ```ts
 * import { customFunction, initSQLite, isOpfsSupported } from '@subframe7536/sqlite-wasm'
 * import { useOpfsStorage } from '@subframe7536/sqlite-wasm/opfs'
 * import { uuidv7 } from 'uuidv7'
 *
 * const { run, sqlite, db } = await initSQLite(
 *   useOpfsStorage('test')
 * )
 * customFunction(sqlite, db, 'uuidv7', () => uuidv7())
 * console.log(await run('select uuidv7() as a'))
 * // [{ "a": "01932f1b-b663-7714-af4d-17a3d9efc7b3" }]
 * ```
 */
declare function customFunction<N extends string, T extends SQLiteCompatibleType[]>(sqlite: SQLiteAPI, db: number, fnName: N, fn: N extends '' ? never : (...args: T) => (SQLiteCompatibleType | number[]) | null, options?: {
    deterministic?: boolean;
    directOnly?: boolean;
    varargs?: boolean;
}): void;
declare function customFunctionCore<N extends string, T extends SQLiteCompatibleType[]>(core: SQLiteDBCore, fnName: N, fn: N extends '' ? never : (...args: T) => (SQLiteCompatibleType | number[]) | null, options?: {
    deterministic?: boolean;
    directOnly?: boolean;
    varargs?: boolean;
}): void;
/**
 * Parse options with existing database
 * @param data database File or ReadableStream
 * @param options extra options
 * @example
 * ```ts
 * import { initSQLite, withExistDB } from '@subframe7536/sqlite-wasm'
 * import { useIdbStorage } from '@subframe7536/sqlite-wasm/idb'
 *
 * const db = initSQLite(
 *   useIdbStorage('test.db', withExistDB(FileOrReadableStream, { url }))
 * )
 * ```
 */
declare function withExistDB<T extends BaseStorageOptions>(data: File | ReadableStream, options?: Omit<T, 'beforeOpen'>): T;
declare function close(core: SQLiteDBCore): Promise<void>;
declare function changes(core: SQLiteDBCore): number | bigint;
declare function lastInsertRowId(core: SQLiteDBCore): number | bigint;
declare function stream(core: SQLiteDBCore, onData: (data: Record<string, SQLiteCompatibleType>) => void, sql: string, parameters?: SQLiteCompatibleType[]): Promise<void>;
declare function run(core: SQLiteDBCore, sql: string, parameters?: SQLiteCompatibleType[]): Promise<Array<Record<string, SQLiteCompatibleType>>>;
declare function iterator(core: SQLiteDBCore, sql: string, parameters?: SQLiteCompatibleType[], chunkSize?: number): AsyncIterableIterator<Record<string, SQLiteCompatibleType>[]>;
declare function parseOpenV2Flag(readonly?: boolean): number;
declare function reopen(core: SQLiteDBCore, readonly?: boolean): Promise<void>;

declare class MemoryVFS {
    static create(name: any, module: any): Promise<MemoryVFS>;
    constructor(name: any, module: any);
    mapNameToFile: Map<any, any>;
    mapIdToFile: Map<any, any>;
    close(): void;
    /**
     * @param {string?} filename
     * @param {number} fileId
     * @param {number} flags
     * @param {DataView} pOutFlags
     * @returns {number|Promise<number>}
     */
    jOpen(filename: string | null, fileId: number, flags: number, pOutFlags: DataView): number | Promise<number>;
    /**
     * @param {number} fileId
     * @returns {number|Promise<number>}
     */
    jClose(fileId: number): number | Promise<number>;
    /**
     * @param {number} fileId
     * @param {Uint8Array} pData
     * @param {number} iOffset
     * @returns {number|Promise<number>}
     */
    jRead(fileId: number, pData: Uint8Array, iOffset: number): number | Promise<number>;
    /**
     * @param {number} fileId
     * @param {Uint8Array} pData
     * @param {number} iOffset
     * @returns {number|Promise<number>}
     */
    jWrite(fileId: number, pData: Uint8Array, iOffset: number): number | Promise<number>;
    /**
     * @param {number} fileId
     * @param {number} iSize
     * @returns {number|Promise<number>}
     */
    jTruncate(fileId: number, iSize: number): number | Promise<number>;
    /**
     * @param {number} fileId
     * @param {DataView} pSize64
     * @returns {number|Promise<number>}
     */
    jFileSize(fileId: number, pSize64: DataView): number | Promise<number>;
    /**
     * @param {string} name
     * @param {number} syncDir
     * @returns {number|Promise<number>}
     */
    jDelete(name: string, syncDir: number): number | Promise<number>;
    /**
     * @param {string} name
     * @param {number} flags
     * @param {DataView} pResOut
     * @returns {number|Promise<number>}
     */
    jAccess(name: string, flags: number, pResOut: DataView): number | Promise<number>;
}

/**
 * Store data in memory,
 * use `MemoryVFS` with `wa-sqlite.wasm`,
 * no data persistence
 * @param options options
 * @example
 * ```ts
 * import { initSQLite, isOpfsSupported, useMemoryStorage } from '@subframe7536/sqlite-wasm'
 *
 * // optional url
 * const url = 'https://cdn.jsdelivr.net/npm/@subframe7536/sqlite-wasm@0.5.0/wa-sqlite.wasm'
 * const url1 = 'https://cdn.jsdelivr.net/gh/subframe7536/sqlite-wasm@v0.5.0/wa-sqlite-fts5/wa-sqlite.wasm'
 *
 * const { run, changes, lastInsertRowId, close } = await initSQLite(
 *   useMemoryStorage({ url })
 * )
 * ```
 */
declare function useMemoryStorage(options?: BaseStorageOptions): Promise<InitSQLiteOptions>;

export { BaseStorageOptions, FacadeVFS, InitSQLiteOptions, MemoryVFS, Promisable, SQLiteDB, SQLiteDBCore, changes, close, customFunction, customFunctionCore, dumpVFS, exportDatabase, exportDatabaseFromFsHandle, exportDatabaseFromIDB, importDatabase, importDatabaseToIdb, initSQLite, initSQLiteCore, isIdbSupported, isModuleWorkerSupport, isOpfsSupported, iterator, lastInsertRowId, parseOpenV2Flag, reopen, run, stream, useMemoryStorage, withExistDB };
