import { B as BaseStorageOptions, I as InitSQLiteOptions } from './index-DoihFDbg.cjs';

declare class OPFSCoopSyncVFS {
    static create(name: any, module: any): Promise<OPFSCoopSyncVFS>;
    constructor(name: any, module: any);
    /** @type {Map<number, File>} */ mapIdToFile: Map<number, File>;
    lastError: null;
    log: null;
    /** @type {Map<string, PersistentFile>} */ persistentFiles: Map<string, PersistentFile>;
    /** @type {Map<string, FileSystemSyncAccessHandle>} */ boundAccessHandles: Map<string, FileSystemSyncAccessHandle>;
    /** @type {Set<FileSystemSyncAccessHandle>} */ unboundAccessHandles: Set<FileSystemSyncAccessHandle>;
    /** @type {Set<string>} */ accessiblePaths: Set<string>;
    releaser: null;
    /**
     * @param {string?} zName
     * @param {number} fileId
     * @param {number} flags
     * @param {DataView} pOutFlags
     * @returns {number}
     */
    jOpen(zName: string | null, fileId: number, flags: number, pOutFlags: DataView): number;
    /**
     * @param {string} zName
     * @param {number} syncDir
     * @returns {number}
     */
    jDelete(zName: string, syncDir: number): number;
    /**
     * @param {string} zName
     * @param {number} flags
     * @param {DataView} pResOut
     * @returns {number}
     */
    jAccess(zName: string, flags: number, pResOut: DataView): number;
    /**
     * @param {number} fileId
     * @returns {number}
     */
    jClose(fileId: number): number;
    /**
     * @param {number} fileId
     * @param {Uint8Array} pData
     * @param {number} iOffset
     * @returns {number}
     */
    jRead(fileId: number, pData: Uint8Array, iOffset: number): number;
    /**
     * @param {number} fileId
     * @param {Uint8Array} pData
     * @param {number} iOffset
     * @returns {number}
     */
    jWrite(fileId: number, pData: Uint8Array, iOffset: number): number;
    /**
     * @param {number} fileId
     * @param {number} iSize
     * @returns {number}
     */
    jTruncate(fileId: number, iSize: number): number;
    /**
     * @param {number} fileId
     * @param {number} flags
     * @returns {number}
     */
    jSync(fileId: number, flags: number): number;
    /**
     * @param {number} fileId
     * @param {DataView} pSize64
     * @returns {number}
     */
    jFileSize(fileId: number, pSize64: DataView): number;
    /**
     * @param {number} fileId
     * @param {number} lockType
     * @returns {number}
     */
    jLock(fileId: number, lockType: number): number;
    /**
     * @param {number} fileId
     * @param {number} lockType
     * @returns {number}
     */
    jUnlock(fileId: number, lockType: number): number;
    /**
     * @param {number} fileId
     * @param {number} op
     * @param {DataView} pArg
     * @returns {number|Promise<number>}
     */
    jFileControl(fileId: number, op: number, pArg: DataView): number | Promise<number>;
    /**
     * @param {Uint8Array} zBuf
     * @returns
     */
    jGetLastError(zBuf: Uint8Array): any;
    #private;
}
declare class File {
    constructor(path: any, flags: any);
    /** @type {string} */ path: string;
    /** @type {number} */ flags: number;
    /** @type {FileSystemSyncAccessHandle} */ accessHandle: FileSystemSyncAccessHandle;
    /** @type {PersistentFile?} */ persistentFile: PersistentFile | null;
}
declare class PersistentFile {
    constructor(fileHandle: any);
    /** @type {FileSystemFileHandle} */ fileHandle: FileSystemFileHandle;
    /** @type {FileSystemSyncAccessHandle} */ accessHandle: FileSystemSyncAccessHandle;
    /** @type {boolean} */ isLockBusy: boolean;
    /** @type {boolean} */ isFileLocked: boolean;
    /** @type {boolean} */ isRequestInProgress: boolean;
    /** @type {function} */ handleLockReleaser: Function;
    /** @type {BroadcastChannel} */ handleRequestChannel: BroadcastChannel;
    /** @type {boolean} */ isHandleRequested: boolean;
}

/**
 * Store data in [OPFS](https://developer.mozilla.org/en-US/docs/Web/API/File_System_API/Origin_private_file_system) through [FileSystemSyncAccessHandle](https://developer.mozilla.org/en-US/docs/Web/API/FileSystemSyncAccessHandle),
 * use `OPFSCoopSyncVFS` with `wa-sqlite.wasm`, smaller and faster than all other persist storages
 *
 * **MUST RUN IN WEB WORKER**
 * @param path db file directory path
 * @param options wasm file url
 * @example
 * ```ts
 * import { initSQLite, isOpfsSupported } from '@subframe7536/sqlite-wasm'
 * import { useOpfsStorage } from '@subframe7536/sqlite-wasm/opfs'
 *
 * // optional url
 * const url = 'https://cdn.jsdelivr.net/npm/@subframe7536/sqlite-wasm@0.5.0/dist/wa-sqlite.wasm'
 *
 * onmessage = async () => {
 *   if (!await isOpfsSupported()) { // this can be called in main thread
 *     return
 *   }
 *   const { run, changes, lastInsertRowId, close } = await initSQLite(
 *     useOpfsStorage('test.db', url)
 *   )
 * }
 * ```
 */
declare function useOpfsStorage(path: string, options?: BaseStorageOptions): Promise<InitSQLiteOptions>;

export { OPFSCoopSyncVFS, useOpfsStorage };
